#!/usr/bin/python3
# See: https://nblock.org/2021/04/09/sync-date-and-time-on-an-offline-fronius-datalogger/
# License: GPLv3
__doc__ = "Set date and time on a Fronius Datalogger"

import argparse
import datetime
import logging
import zoneinfo
from email.utils import parsedate_to_datetime

import requests
from requests.auth import HTTPDigestAuth


class HTTPDigestAuthFronius(HTTPDigestAuth):
    def handle_401(self, r, **kwargs):
        # Replace www-authenticate unconditionally
        r.headers["www-authenticate"] = r.headers.get("x-www-authenticate")
        return super().handle_401(r, **kwargs)


def parse_args():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "-v",
        "--verbosity",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        default="INFO",
        help="the log verbosity level",
    )
    parser.add_argument(
        "-u",
        "--url",
        default="http://192.168.250.181",
        help="the URL of the Fronius Datalogger",
    )
    parser.add_argument(
        "-U",
        "--username",
        default="admin",
        choices=["admin", "service"],
        help="the username to authenticate as",
    )
    parser.add_argument(
        "-P",
        "--password",
        help="the password",
    )
    parser.add_argument(
        "-t",
        "--timezone",
        default="Europe/Vienna",
        help="the timezone to set on the device",
    )
    parser.add_argument(
        "-d",
        "--difference",
        type=int,
        default=3,
        help="a time difference bigger than this cause the time to be synchronized.",
    )

    return parser.parse_args()


def main():
    args = parse_args()
    logging.basicConfig(level=args.verbosity)

    tz = zoneinfo.ZoneInfo(args.timezone)
    session = requests.Session()

    r = session.get(args.url)
    r.raise_for_status()
    date_raw = r.headers.get("date")
    parsed = parsedate_to_datetime(date_raw).astimezone(tz)
    now = datetime.datetime.now(tz).replace(microsecond=0)
    logging.debug(f"Remote timestamp: {parsed}")
    logging.debug(f"Local timestamp: {now}")

    diff = parsed.timestamp() - now.timestamp()
    if abs(diff) > args.difference:
        logging.info(f"Time is off by {diff} seconds - sync with local time")

        r = session.post(
            f"{args.url}/config/date/",
            params={"method": "save"},
            json={
                "date": {
                    "time": int(now.timestamp()),
                    "timezone": tz.key,
                    "timesync": False,
                }
            },
            auth=HTTPDigestAuthFronius(args.username, args.password),
        )
        r.raise_for_status()


if __name__ == "__main__":
    main()
